@@ -1,30 +1,32 @@
 use std::convert::Infallible;
-use std::env;
 use std::net::SocketAddr;
-use std::num::ParseIntError;
 
 use hyper::service::{make_service_fn, service_fn};
 use hyper::header::{HeaderValue, CONTENT_LENGTH};
 use hyper::{Body, Method, Request, Response, Server, StatusCode};
 
+use structopt::StructOpt;
+
 #[derive(PartialEq, Debug)]
 struct ParseError {}
 
-#[derive(PartialEq, Debug)]
-enum InvalidArgument {
-    Parse(ParseError),
-    ParseInt(ParseIntError)
-}
+/// Find acronym meaning.
+#[derive(Debug, StructOpt)]
+#[structopt(name = "args", about = "Provide echo server configuration")]
+struct Cli {
+    /// The acronym to search for
+    #[structopt(short, long)]
+    port: Option<u16>,
 
-impl From<ParseIntError> for InvalidArgument {
-    fn from(err: ParseIntError) -> Self {
-        InvalidArgument::ParseInt(err)
-    }
+    /// Context to search in
+    #[structopt(short, long)]
+    body: Option<String>,
 }
 
 #[tokio::main]
 async fn main() {
-    let port = get_port(&env::args().collect::<Vec<String>>()).unwrap_or(8080);
+    let args = Cli::from_args();
+    let port = args.port.unwrap_or(8080);
     let address = SocketAddr::from(([0, 0, 0, 0], port));
     let server = Server::bind(&address).serve(make_service_fn(|_server| async {
         Ok::<_, Infallible>(service_fn(handle_request))
@@ -43,40 +45,21 @@ async fn main() {
     }
 }
 
-fn get_argument(name: &str, args: &[String]) -> Option<String> {
-    match args.iter().find(|arg| arg.contains(name)) {
-        Some(arg) => arg.split('=').map(|v| v.to_owned()).collect::<Vec<String>>().pop(),
-        _ => None
-    }
-}
-
-fn get_port(args: &[String]) -> Result<u16, InvalidArgument> {
-    let port = get_argument("--port", args);
-
-    match port {
-        Some(p) => Ok(p.parse::<u16>()?),
-        None => Err(InvalidArgument::Parse(ParseError {}))
-    }
-}
-
-fn get_body(args: &[String]) -> Result<String, InvalidArgument> {
-    get_argument("--body", args)
-        .ok_or(InvalidArgument::Parse(ParseError {}))
-}
-
 async fn handle_request(req: Request<Body>) -> Result<Response<Body>, Infallible> {
     let mut response = Response::new(Body::empty());
     let echo_headers = response.headers_mut();
     let headers = req.headers();
 
-    // Echo HTTP headers`
+    // Echo HTTP headers
     headers.iter().for_each(|(name, value)| {
         echo_headers.insert(name, value.clone());
     });
 
+    // Handle each HTTP verb
     match (req.method(), req.uri().path()) {
         (&Method::GET, "/") => {
-            let body = get_body(&env::args().collect::<Vec<String>>()).unwrap_or_else(|_| "".to_owned());
+            let args = Cli::from_args();
+            let body = args.body.unwrap_or_else(|| String::from(""));
             let content_length = HeaderValue::from_str(
                 &body.as_bytes().len().to_string()
             ).unwrap_or_else(|_| HeaderValue::from_static("0"));
@@ -110,23 +93,8 @@ mod test {
 
     #[test]
     fn test_get_body() {
-        let args = ["--body=hello world", "body=bad", "--body=42"].map(|v| v.to_string());
+        let resp = tokio_test::block_on(handle_request(Request::new(Body::from("hello world"))));
 
-        assert_eq!(get_body(&args[..]), Ok("hello world".to_owned()));
-        assert_eq!(get_body(&args[1..]), Ok("42".to_owned()));
-    }
-
-    #[test]
-    fn test_get_port() {
-        let args = ["--port=1337", "--port=8081"].map(|v| v.to_string());
-
-        assert_eq!(get_port(&args[..]), Ok(1337));
-        assert_eq!(get_port(&args[1..]), Ok(8081));
-    }
-
-    #[test]
-    fn test_get_argument() {
-        let args = ["--port=8080", "port=8081"].map(|v| v.to_string());
-        assert_eq!(get_argument("--port", &args[..]), Some("8080".to_string()));
+        assert_eq!(resp.unwrap().status(), StatusCode::OK);
     }
 }
\ No newline at end of file
